unit Main;

interface

uses
  Windows, SysUtils, Classes, Graphics, Controls, Forms, StdCtrls, ExtCtrls,
  ComCtrls, Dialogs;

type
  TMainForm = class(TForm)
    SaveDialog1: TSaveDialog;
    outputMemo: TMemo;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    DirA: TEdit;
    DirA_Change: TButton;
    Label3: TLabel;
    DirB: TEdit;
    DirB_Change: TButton;
    Panel2: TPanel;
    SaveBtn: TButton;
    chkShowVorkommen: TCheckBox;
    chkShowFehlende: TCheckBox;
    chkSubdirectories: TCheckBox;
    Panel3: TPanel;
    StartBtn: TButton;
    Button1: TButton;
    procedure StartBtnClick(Sender: TObject);
    procedure SaveBtnClick(Sender: TObject);
    procedure DirA_ChangeClick(Sender: TObject);
    procedure DirB_ChangeClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

{$R WindowsXP.res}

uses
  HighPerfFileComparator, BrowseFolder;

resourcestring
  LNG_SELECT_FOLDER = 'Ordner auswhlen';

procedure ListFiles(Directory: string; list: TStringList; recursive: boolean);
var
  SR: TSearchRec;
begin
  Directory := IncludeTrailingPathDelimiter(Directory);

  if Application.Terminated then exit;
  Application.ProcessMessages;

  if FindFirst(Directory+'*', faAnyFile, SR) = 0 then;
  begin
    repeat
      if (SR.Name <> '.') and (SR.Name <> '..') then
      begin
        if SR.Attr and faDirectory = faDirectory then
        begin
          if recursive then
          begin
            ListFiles(Directory + SR.Name, list, recursive);
          end;
        end
        else
        begin
          list.Add(Directory + SR.Name);
        end;
      end;
    until FindNext(SR) <> 0;
  end;
  FindClose(SR);
end;

resourcestring
  LNG_NO_OCCOURENCE = '"%s" kein Vorkommen in "%s"';
  LNG_PLEASE_ENTER_DIFFERENT_DIRECTORIES =
    'Bitte geben Sie zwei unterschiedliche Verzeichnisse an!';
  LNG_DIR_X_NOT_EXIST = 'Verzeichnis %d existiert nicht!';
  LNG_CALC_DIR_LIST = 'Bereche Verzeichnisinhalte...';
  LNG_EQUAL = '"%s" = "%s"';

procedure TMainForm.StartBtnClick(Sender: TObject);
var
  filesA, filesB: TStringList;
  i, j: integer;
  fc: THighPerfFileComparator;
  somethingfound: boolean;
  dir_a, dir_b: string;
begin          
  dir_a := IncludeTrailingPathDelimiter(ExpandUNCFileName(DirA.Text));
  dir_b := IncludeTrailingPathDelimiter(ExpandUNCFileName(DirB.Text));

  if dir_a = dir_b then
  begin
    showmessage(LNG_PLEASE_ENTER_DIFFERENT_DIRECTORIES);
    exit;
  end;
  if not DirectoryExists(dir_a) then
  begin
    ShowMessageFmt(LNG_DIR_X_NOT_EXIST, [1]);
    exit;
  end;
  if not DirectoryExists(dir_b) then
  begin
    ShowMessageFmt(LNG_DIR_X_NOT_EXIST, [2]);
    exit;
  end;

  StartBtn.Enabled := false;
  try
    try
      outputMemo.Lines.Add(LNG_CALC_DIR_LIST);
      outputMemo.Lines.Add('');
      application.ProcessMessages;

      fc := THighPerfFileComparator.Create;
      // fc.ClearCache;
      try
        filesA := TStringList.Create;
        try
          ListFiles(dir_a, filesA, chkSubdirectories.Checked);
          filesB := TStringList.Create;
          try
            ListFiles(dir_b, filesB, chkSubdirectories.Checked);

            outputMemo.Clear;

            for i := 0 to filesA.Count - 1 do
            begin
              somethingfound := false;

              Application.ProcessMessages;
              if Application.Terminated then exit;

              for j := 0 to filesB.Count - 1 do
              begin
                Application.ProcessMessages;
                if Application.Terminated then exit;

                if fc.Compare(filesA.Strings[i], filesB.Strings[j]) then
                begin
                  somethingfound := true;
                  if chkShowVorkommen.Checked then
                  begin
                    outputMemo.Lines.Add(Format(LNG_EQUAL,
                      [filesA.Strings[i], filesB.Strings[j]]));
                    application.ProcessMessages;
                  end;
                end;
              end;

              if not somethingfound then
              begin
                if chkShowFehlende.Checked then
                begin
                  outputMemo.Lines.Add(Format(LNG_NO_OCCOURENCE,
                    [filesA.Strings[i], DirB.Text]));
                  application.ProcessMessages;
                end;
              end;
            end;
          finally
            filesB.Free;
          end;
        finally
          filesA.Free;
        end;
      finally
        fc.Free;
      end;
    finally
      StartBtn.Enabled := true;
    end;
  except
    StartBtn.Enabled := true;
  end;
end;

procedure TMainForm.SaveBtnClick(Sender: TObject);
begin
  if savedialog1.Execute then
  begin
    outputMemo.lines.SaveToFile(savedialog1.FileName);
  end;
end;

procedure TMainForm.DirA_ChangeClick(Sender: TObject);
var
  fol: string;
begin
  fol := BrowseForFolder(LNG_SELECT_FOLDER, false);
  if fol <> '' then DirA.Text := IncludeTrailingPathDelimiter(fol);
end;

procedure TMainForm.DirB_ChangeClick(Sender: TObject);
var
  fol: string;
begin
  fol := BrowseForFolder(LNG_SELECT_FOLDER, false);
  if fol <> '' then DirB.Text := IncludeTrailingPathDelimiter(fol);
end;

procedure TMainForm.Button1Click(Sender: TObject);
var
  tmp: string;
begin
  tmp := DirB.Text;
  DirB.Text := DirA.Text;
  DirA.Text := tmp;
end;

end.
